
// The code below uses require.js, a module system for javscript:
// http://requirejs.org/docs/api.html#define
//
// You don't have to use require.js, and you can delete all of this if
// you aren't (make sure to uncomment the script tags in index.html also)


// Set the path to jQuery, which will fall back to the local version
// if google is down
require.config({
  baseUrl: 'js/lib',
  paths: {
    'jquery':
            ['//ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min',
             'lib/jquery']
  }
});

var global = this;

require(['jquery', 'crafty'], function ($) {

  // -----------------------------------------------------------------
  // Configuration
  // -----------------------------------------------------------------

  var RUN = {
    'width' : 572,
    'height' : 120,
    'player' : null,
    'life' : 3,
    'score' : 0,
    'sprites' : {
      'src' : 'img/sprites.png',
      'size' : 1
    },
    'entities' : {
      'star' : {
        'attr' : {
          'y' : 20,
          'speed' : 5
        },
        'spawn' : {
          'min' : 600,
          'max' : 2000,
          'next' : null
        },
        'sprite' : {
          'size' : 16,
          'coords' : [0, 152, 16, 16]
        }
      },
      'enemy' : {
        'attr' : {
          'y' : 84,
          'speed' : 5
        },
        'spawn' : {
          'min' : 400,
          'max' : 2000,
          'next' : null
        },
        'sprite' : {
          'size' : 16,
          'coords' : [16, 152, 16, 16]
        }
      },
      'player' : {
        'sprite' : {
          'size' : 32,
          'coords' : [0, 120, 32, 32],
          'anim' : [ [0, 120], [32, 120], [64, 120], [96, 120], [128, 120],
                     [160, 120], [192, 120], [224, 120], [256, 120],
                     [288, 120], [320, 120], [352, 120], [384, 120],
                     [416, 120], [448, 120], [480, 120]
                   ]
        }
      },
      'floor' : {
        'sprite' : {
          'size' : 20,
          'coords' : [0, 170, 600, 16]
        }
      },
      'sky' : {
        'sprite' : {
          'coords' : [0, 0, 600, 120]
        }
      }
    }
  };

  // Init the game
  Crafty.init(RUN.width, RUN.height);
  Crafty.canvas.init();

  // -----------------------------------------------------------------
  // Components
  // -----------------------------------------------------------------

  /**
   * Twoway component but the caracter goes faster left than right.
   * This is used to simulate the moving ground.
   */
  Crafty.c("TwowayRunning", {
    _speed: 3,
    _up: false,

    init: function () {
      this.requires("Keyboard");
    },

    twoway: function (speed,jump) {
      if(speed) this._speed = speed;
      jump = jump || this._speed * 2;

      this.bind("EnterFrame", function () {
        if (this.disableControls) return;
        if(this.isDown("RIGHT_ARROW") || this.isDown("D")) {
          this.x += this._speed;
        }
        if(this.isDown("LEFT_ARROW") || this.isDown("A")) {
          this.x -= this._speed * 2;
        }
        if(this._up) {
          this.y -= jump;
          this._falling = true;
        }
      }).bind("KeyDown", function () {
        if(this.isDown("UP_ARROW") || this.isDown("W")) this._up = true;
      });

      return this;
    }
  });

  /**
   * Inside component to make sure an entity cannot go outside the stage.
   */
  Crafty.c("Inside", {
    init: function () {
      this.bind("EnterFrame", function () {
        if (this.x < 0) {
          this.x = 0;
        }
        if (this.x > Crafty.viewport.width - this.w) {
          this.x = Crafty.viewport.width - this.w
        }

        return this;
      });
    }
  });

  /**
   * Moving component for entities moving from right to left.
   */
  Crafty.c("Moving", {
    init: function () {
      this.requires("2D");

      this.speed = 5;

      this.bind("EnterFrame", function () {
        // Move this to the left
        this.x -= this.speed;

        // If this is outside the stage, destroy it
        if (this.x <= - this.width) {
          this.destroy();
        }
      });
    },

    moving: function (speed) {
      this.speed = speed;
    }
  });

  Crafty.c("Enemy", {
    init: function () {
      this.requires("Collision");

      this.collision()
          .onHit("player", function () {
            RUN.player.hurt(1);
            this.destroy();
          });
    }
  });

  Crafty.c("Star", {
    init: function () {
      this.requires("Collision");

      this.collision()
          .onHit("player", function () {
            RUN.score += 1;
            this.destroy();
          });
    }
  });

  Crafty.c("Health", {
    init: function () {
      this.health = RUN.life;
    },

    hurt: function (aouch) {
      this.health -= aouch;
      if (this.health <= 0) {
        gameOver();
      }
    }
  });

  // -----------------------------------------------------------------
  // Functions
  // -----------------------------------------------------------------

  function gameOver() {
    // Stop spawning
    clearInterval(RUN.entities.star.spawn.next);
    clearInterval(RUN.entities.enemy.spawn.next);

    // Go to game over scene
    Crafty.scene("gameover");
  }

  function createSprites() {
    // Splice the RUN sprite
    Crafty.sprite(RUN.sprites.size, RUN.sprites.src, {
      'sky' : RUN.entities.sky.sprite.coords,
      'floor' : RUN.entities.floor.sprite.coords,
      'player' : RUN.entities.player.sprite.coords,
      'enemy' : RUN.entities.enemy.sprite.coords,
      'star' : RUN.entities.star.sprite.coords
    });

    // Start the main scene when loaded
    Crafty.scene("start");
  }

  // -----------------------------------------------------------------
  // Scenes
  // -----------------------------------------------------------------

  // Start screen
  Crafty.scene("start", function () {
    Crafty.e("2D, Canvas, sky")
          .attr({ x: 0, y: 0 });

    Crafty.e("2D, DOM, Text")
          .text('<a href="#" id="play" onclick="Crafty.scene(\'main\'); return false;">Play</a>');
  });

  // Game over screen
  Crafty.scene("gameover", function () {
    Crafty.e("2D, Canvas, sky")
          .attr({x: 0, y: 0});

    Crafty.e("2D, DOM, Text").text('<p><a href="#" id="play-again" onclick="Crafty.scene(\'main\'); return false;">Play again!</a></p>');
    Crafty.e("2D, DOM, Text")
          .css("color", "red")
          .text('<p id="life">Life: ' + RUN.player.health + '</p>');
    Crafty.e("2D, DOM, Text")
          .text('<p id="score">' + RUN.score + '</p>');
  });

  // Playing screen
  Crafty.scene("main", function () {
    RUN.score = 0;

    Crafty.e("2D, Canvas, SpriteAnimation, sky")
          .attr({x: 0, y: 0})
          .animate("movingsky", 0, 0, 599)
          .animate("movingsky", 600, -1);

    RUN.player = Crafty.e("2D, Canvas, SpriteAnimation, TwowayRunning, Gravity, Inside, Collision, Health, player")
                       .attr({
                         x: 0,
                         y: Crafty.viewport.height - RUN.entities.floor.sprite.size - RUN.entities.player.sprite.size
                       })
                       .animate("walk", RUN.entities.player.sprite.anim)
                       .twoway(0, 5)
                       .gravity("floor");

    RUN.player.animate("walk", 10, -1);

    // Life text
    Crafty.e("2D, DOM, Text")
          .css("color", "red")
          .bind("EnterFrame", function () {
            this.text('<p id="life">Life: ' + RUN.player.health + '</p>');
          });
    // Score text
    Crafty.e("2D, DOM, Text")
          .bind("EnterFrame", function () {
            this.text('<p id="score">' + RUN.score + '</p>');
          });

    var coords = [];
    var floorSpeed = 10;
    for (var i = 0; i < 600; i += floorSpeed) {
      coords.push([i, 170]);
    }
    // Floor (mainly for gravity)
    Crafty.e("2D, Canvas, SpriteAnimation, floor")
          .attr({
            x: 0,
            y: Crafty.viewport.height - RUN.entities.floor.sprite.size,
            w: Crafty.viewport.width,
            h: RUN.entities.floor.sprite.size
          })
          .animate("movingfloor", coords)
          .animate("movingfloor", 5, -1);

    // Spawn a new enemy recursively, with a random interval between each
    function spawn(type, components) {
      var entity = RUN.entities[type];
      clearInterval(entity.spawn.next);
      entity.spawn.next = setInterval(
        function () {
          Crafty.e("Canvas, Moving, " + components)
                .attr(entity.attr)
                .attr({ x: RUN.width });
          spawn(type, components);
        },
        Math.floor(Math.random() * (entity.spawn.max - entity.spawn.min + 1)) + entity.spawn.min
      );
    }

    // Start generating enemies and stars
    spawn("enemy", "Enemy, enemy");
    spawn("star", "Star, star");
  });

  // Start loading the game
  createSprites();
});

// END REQUIRE.JS CODE
// Remove all of this if not using require.js
